import logging
from typing import Dict

from sqlalchemy.orm import Session

from models import Author

logger = logging.getLogger(__name__)

def get_or_create_author(session: Session, author_data: Dict[str, str]) -> Author:
    """
    Checks if an author exists in the database, creates one if not.

    :param session: SQLAlchemy session object.
    :param author_data: Dictionary containing author's first and last name.
    :return: An Author instance (either existing or newly created).
    """
    existing_author = session.query(Author).filter_by(
        first_name=author_data["first_name"],
        last_name=author_data["last_name"]
    ).one_or_none()

    if existing_author is not None:
        logger.debug(f"Author {author_data['first_name']} {author_data['last_name']} already exists. Reusing.")
        return existing_author

    logger.debug(f"Creating new author: {author_data['first_name']} {author_data['last_name']}")
    author = Author(first_name=author_data["first_name"], last_name=author_data["last_name"])
    session.add(author)
    return author

__all__ = ["get_or_create_author"]
