import os
import logging
import time

from typing import List, Dict

from xml.etree import ElementTree as ET
from xmlschema import XMLSchema
from sqlalchemy.exc import IntegrityError

from database.manager import DatabaseManager
from database.utils import get_or_create_categories, get_or_create_author
from utils.errors.import_error.xsd_scheme_not_found import XsdSchemeNotFoundError
from utils.errors.import_error.invalid_contents_error import InvalidContentsError
from utils.errors.import_error.import_error import ImportError
from utils.config import UserConfig
from models import Book, Author, BookCategory

class BookImporter:
    def __init__(self):
        # Initialize the logger and schema
        self.logger = logging.getLogger(__name__)
        try:
            self.logger.debug("Opening XSD scheme in ./")
            scheme_path = os.path.join(os.path.dirname(__file__), "book_import_scheme.xsd")
            self.schema = XMLSchema(scheme_path)
        except Exception as e:
            self.logger.error("Failed to load XSD scheme")
            raise XsdSchemeNotFoundError(f"Failed to load XSD schema: {e}")

    def parse_xml(self, file_path: str) -> List[Dict[str, object]]:
        """Parses the XML file and validates it against the XSD schema."""
        try:
            tree = ET.parse(file_path)
            root = tree.getroot()

            if not self.schema.is_valid(file_path):
                raise InvalidContentsError("XML file is not valid according to XSD schema.")

            books = []
            for book_element in root.findall("book"):
                title = book_element.find("title").text
                year_published = book_element.find("year_published").text
                description = book_element.find("description").text
                isbn = book_element.find("isbn").text

                # Parse author
                author_element = book_element.find("author")
                author = {
                    "first_name": author_element.find("first_name").text,
                    "last_name": author_element.find("last_name").text
                }

                # Parse categories
                category_elements = book_element.find("categories").findall("category")
                categories = [category_element.text for category_element in category_elements]

                # Create a book dictionary with explicit types
                book = {
                    "title": title,
                    "description": description,
                    "year_published": year_published,
                    "isbn": isbn,
                    "author": author,
                    "categories": categories
                }
                books.append(book)

            return books
        except ET.ParseError as e:
            raise ImportError(f"Failed to parse XML file: {e}")