import os
import logging
from typing import Optional, List, Dict
import xml.etree.ElementTree as ET
from xml.dom import minidom
from xmlschema import XMLSchema

from utils.errors import (
    NoExportEntityError,
    ExportError,
    ExportFileError,
    InvalidContentsError,
    XsdSchemeNotFoundError,
    ImportError,
)

# Initialize logger and XML Schema
logger = logging.getLogger(__name__)

from models import BooksOverview

from database import fetch_all_book_overviews

def export_to_xml(file_path: str) -> None:
    logger.debug("Attempting to export book overview")
    all_books = fetch_all_book_overviews()

    if not all_books:
        logger.warning("No books found to export")
        raise NoExportEntityError("No books found to export")

    xml = overviews_to_xml(all_books)

    try:
        with open(file_path, "w", encoding="utf-8") as file:
            file.write(xml)
            logger.info("Successfully saved book overview export")
    except OSError as e:
        raise ExportFileError("Failed to save to a file") from e


def overviews_to_xml(overview_list: List[BooksOverview]) -> str:
    root = ET.Element("book_overview")

    for book_overview in overview_list:
        # Create a <book_entry> element
        book_element = ET.SubElement(root, "book_entry")

        # Add <title>
        title_element = ET.SubElement(book_element, "title")
        title_element.text = book_overview.title

        # Add <author>
        author_element = ET.SubElement(book_element, "author")
        author_element.text = book_overview.author_name

        # Add <year_published>
        year_published_element = ET.SubElement(book_element, "year_published")
        year_published_element.text = book_overview.year_published

        # Add <isbn>
        isbn_element = ET.SubElement(book_element, "isbn")
        isbn_element.text = book_overview.isbn

        # Add <borrower_name>
        borrower_name = ET.SubElement(book_element, "borrower_name")
        borrower_name.text = book_overview.borrower_name

        # Add <librarian_name>
        librarian_name = ET.SubElement(book_element, "librarian_name")
        librarian_name.text = book_overview.librarian_name

    # Convert the tree to a string
    tree_str = ET.tostring(root, encoding="unicode")

    # Pretty print the XML
    pretty_xml = minidom.parseString(tree_str).toprettyxml(indent=(" " * 4))
    return pretty_xml
